@@ -4,17 +4,14 @@ from collections import namedtuple
 from dataclasses import asdict, dataclass, field, fields
 import datetime
 import logging
-from pathlib import Path
 import random
 import re
 from typing import Any, Callable, Collection, Dict, Generator, Generic, Iterable, List, Tuple, Type, TypeVar, Union, \
     get_type_hints
 
 from hppfcl import hppfcl
-from matplotlib import pyplot as plt
 import numpy as np
 import pinocchio as pin
-from roboticstoolbox.tools.trajectory import mstraj
 
 import timor.utilities.errors as err
 from timor.utilities.schema import DEFAULT_DATE_FORMAT
@@ -193,8 +190,8 @@ class TypedHeader:
     @staticmethod
     def cast(value: Any, cast_to_type: Type) -> Any:
         """Casts the value to the dtype"""
-        if cast_to_type is datetime.datetime:
-            return TypedHeader.cast_to_datetime(value)
+        if cast_to_type is datetime.datetime or cast_to_type is datetime.date:
+            return TypedHeader.cast_to_date(value)
         try:
             return cast_to_type(value)
         except TypeError:
@@ -202,13 +199,15 @@ class TypedHeader:
             return value
 
     @staticmethod
-    def cast_to_datetime(value: Union[str, datetime.datetime]) -> datetime.datetime:
-        """Casts the value to a datetime.datetime object"""
-        if isinstance(value, datetime.datetime):
+    def cast_to_date(value: Union[str, datetime.date, datetime.datetime]) -> datetime.date:
+        """Casts the value to a datetime.date object"""
+        if isinstance(value, datetime.date):
             return value
+        if isinstance(value, datetime.datetime):
+            return value.date()
         if isinstance(value, str):
             value = re.sub('/', '-', value)  # Allows parsing yyyy/mm/dd formatted dates
-            return datetime.datetime.strptime(value, DEFAULT_DATE_FORMAT)
+            return datetime.datetime.strptime(value, DEFAULT_DATE_FORMAT).date()
         raise TypeError(f"Cannot cast {value} to datetime.datetime")
 
     @staticmethod
@@ -267,180 +266,6 @@ class TorqueInput:
         assert len(np.unique(self.t)) == len(self.t), "There cannot be duplicate time steps in a Torque Input"
 
 
-@dataclass
-class Trajectory:
-    """Holds trajectory data and makes sure that input arguments are valid"""
-
-    t: Union[np.ndarray, float]
-    q: np.ndarray
-    goals: Dict[str, float] = field(default_factory=lambda: dict())  # Maps goals to the point in time they are achieved
-    dq: np.ndarray = None
-    ddq: np.ndarray = None
-
-    def __post_init__(self):
-        """Enforce typing"""
-        self.t = np.asarray(self.t)
-        self.q = np.asarray(self.q)
-
-        """Validity checks"""
-        if self.t.size == 1:  # Equidistant time steps
-            if len(self.q.shape) == 2:
-                steps = self.q.shape[0]
-                self.t = np.arange(steps, dtype=float) * self.t
-
-        assert np.unique(self.t).size == self.t.size, "There cannot be duplicate time steps in a Trajectory"
-
-        if (self.dq is None) or (self.ddq is None):
-            try:
-                time_delta = self.t[1:] - self.t[:-1]
-                init = np.zeros((1, self.q.shape[1]), dtype=float)
-            except IndexError as exe:
-                if len(self.q.shape) == 1:
-                    init = np.zeros((self.q.shape[0],), dtype=float)
-                    self.dq = init
-                    self.ddq = init
-                else:
-                    raise exe
-
-        if self.dq is not None:
-            self.dq = np.asarray(self.dq)
-            assert self.q.shape == self.dq.shape, \
-                f"Invalid trajectory, shape of q {self.q.shape} and dq {self.dq.shape} do not match."
-        else:
-            # Approximate velocity TODO : Use numpy gradient?
-            dq = (self.q[1:] - self.q[:-1]) / time_delta[:, None]
-            self.dq = np.vstack([init, dq])
-
-        if self.ddq is not None:
-            self.ddq = np.asarray(self.ddq)
-            assert self.q.shape == self.ddq.shape, \
-                f"Invalid trajectory, shape of q {self.q.shape} and ddq {self.ddq.shape} do not match."
-        else:
-            # Approximate acceleration TODO : Use numpy gradient?
-            ddq = (self.dq[1:] - self.dq[:-1]) / time_delta[:, None]
-            self.ddq = np.vstack([init, ddq])
-
-    @staticmethod
-    def from_mstraj(via_points: np.array, dt: float, t_acc: float, qd_max: float, **kwargs) -> 'Trajectory':
-        """
-        Create multi-segment multi-axis trajectory.
-
-        See: https://petercorke.github.io/robotics-toolbox-python/arm_trajectory.html?highlight=mstraj
-
-        :param via_points: Via points to connect with PTP trajectory. Start at first row, stop at last
-        :param dt: sample time in seconds
-        :param t_acc: acceleration time per PTP segment
-        :param qd_max: maximum speed in each axis
-        :param kwargs: Additional key word arguments piped directly into the mstraj function
-        """
-        t = mstraj(via_points, dt, t_acc, qd_max, **kwargs)
-        dq = np.gradient(t.q, axis=0)
-        ddq = np.gradient(dq, axis=0)
-
-        return Trajectory(t=t.t, q=t.q, dq=dq, ddq=ddq, goals={})
-
-    @staticmethod
-    def stationary(configuration: np.array, time: float) -> 'Trajectory':
-        """
-        Create a trajectory that stays in place for time seconds.
-
-        :param configuration: Configuration to stay in.
-        :param time: Time period to stay at this configuration.
-        :return: The stationary trajectory, composed of two configurations and a zero velocity / acceleration
-        """
-        q = np.tile(configuration, (2, 1))
-        dq = np.zeros_like(q)
-        ddq = np.zeros_like(q)
-        return Trajectory(t=np.asarray([0, time]), q=q, dq=dq, ddq=ddq, goals={})
-
-    def to_json_data(self) -> Dict:
-        """Return trajectory as a dictionary that can be serialized as json."""
-        return {
-            "t": self.t.tolist(),
-            "q": self.q.tolist(),
-            "dq": self.dq.tolist(),
-            "ddq": self.ddq.tolist(),
-            "goal2time": self.goals
-        }
-
-    def plot(self, show_goals_reached: bool = False) -> plt.Figure:
-        """
-        Plot this trajectory
-
-        :param show_goals_reached: If set to true, the time at which goals are reached is highlighted in the plot
-        :return: Figure with a plot of this trajectory.
-        """
-        f, (ax1, ax2, ax3) = plt.subplots(3, 1, sharex='all')
-        ax1.plot(self.t, self.q)
-        ax2.plot(self.t, self.dq)
-        ax3.plot(self.t, self.ddq)
-
-        ax1.set(ylabel="Configuration q")
-        ax2.set(ylabel="Velocity dq")
-        ax3.set(xlabel="Time [s]", ylabel="Acceleration ddq")
-
-        if show_goals_reached:
-            t_offset = 0.01 * (max(self.t) - min(self.t))
-            for goal_id, goal_time in self.goals.items():
-                for ax in (ax1, ax2, ax3):
-                    ax.axvline(goal_time, c='blue')
-                    ax.text(goal_time + t_offset, .9, goal_id, c='blue', transform=ax.get_xaxis_transform())
-
-        plt.tight_layout()
-        plt.show()
-
-        return f
-
-    def __add__(self, other: 'Trajectory'):
-        """Append another trajectory to self"""
-        if not isinstance(other, Trajectory):
-            return NotImplemented
-        if (other.t.size == 0) or (self.t.size == 0):
-            return self
-        elif self.t.size > 1:
-            # dt is the offset between end of self and start of other
-            dt = self.t[-1] - self.t[-2]
-        else:
-            if other.t.size < 2:
-                raise ValueError("Adding trajectories needs to deduce step between trajectories "
-                                 "and therefore needs at least one with length > 1")
-            dt = other.t[-1] - other.t[-2]
-
-        offset_other = dt + self.t[-1]
-        t_new = np.hstack((self.t, other.t + offset_other))
-        q_new = np.vstack((self.q, other.q))
-        # Offset the second trajectories goals
-        goals_new = {**self.goals, **{k: v + offset_other for k, v in other.goals.items()}}
-
-        dq_new = np.vstack((self.dq, other.dq))
-        ddq_new = np.vstack((self.ddq, other.ddq))
-
-        return Trajectory(t=t_new, q=q_new, goals=goals_new, dq=dq_new, ddq=ddq_new)
-
-    def __getitem__(self, item) -> Trajectory:
-        """Allows to index a trajectory like one would index every single item of the trajectory"""
-        return self.__class__(
-            t=self.t[item],
-            q=self.q[item],
-            goals={k: v for k, v in self.goals.items() if (np.min(self.t[item]) <= v and v <= np.max(self.t[item]))},
-            dq=self.dq[item],
-            ddq=self.ddq[item]
-        )
-
-    def __eq__(self, other) -> bool:
-        """Test if two trajectories are close together (max distance < 1e-5)"""
-        if type(self) is type(other):
-            return all((np.allclose(self.t, other.t),
-                        np.allclose(self.q, other.q),
-                        np.allclose(self.dq, other.dq),
-                        np.allclose(self.ddq, other.ddq),
-                        self.goals.keys() == other.goals.keys(),
-                        all(abs(self.goals[k] - other.goals[k]) < 1e-5 for k in self.goals.keys())))
-        elif isinstance(other, Trajectory):
-            raise NotImplementedError(f"Comparison of {type(self)} and {type(other)} not yet implemented.")
-        return NotImplemented
-
-
 def deep_iter_dict(d: Dict[any, any], max_depth=10, _prev=None) -> Generator[Tuple[List[any], any], None, None]:
     """
     Iterators over hierarchical dictionaries
@@ -544,13 +369,6 @@ def hppfcl_collision2pin_geometry(geometry: hppfcl.CollisionGeometry,
         return pin.GeometryObject(name, parent_frame, parent_joint, geometry, placement)
 
 
-def map2path(p: Union[str, Path]) -> Path:
-    """Maps strings to paths, but does nothing else s.t. e.g. Django paths are not casted."""
-    if isinstance(p, str):
-        return Path(p)
-    return p
-
-
 def possibly_nest_as_list(value: any, tuple_ok: bool = True):
     """
     A utility to wrap a value within a list if it is not already.
