@@ -179,7 +179,9 @@ class SerializationTests(unittest.TestCase):
         self.assets = file_locations.get_test_tasks()["asset_dir"]
         self.task_files = file_locations.get_test_tasks()['task_files']
         panda_loc = file_locations.robots['panda']
-        self.robot = Robot.PinRobot.from_urdf(panda_loc.joinpath('urdf').joinpath('panda.urdf'), panda_loc.parent)
+        self.panda_urdf = panda_loc.joinpath('urdf').joinpath('panda.urdf')
+        self.panda_assets = panda_loc.parent
+        self.robot = Robot.PinRobot.from_urdf(self.panda_urdf, self.panda_assets)
 
         # Set up some Obstacles
         random_homogeneous = Transformation.random()
@@ -279,6 +281,7 @@ class SerializationTests(unittest.TestCase):
 
         for constraint in self.constraints:
             as_json = json.dumps(constraint.to_json_data())
+
             new = Constraints.ConstraintBase.from_json_data(json.loads(as_json))
             # Also ensure 1-time readable Iterators are parsed
             from_string = Constraints.ConstraintBase.from_json_string(constraint.to_json_string())
@@ -322,6 +325,7 @@ class SerializationTests(unittest.TestCase):
 
         solution_dict = solution.to_json_data()
         self.assertIsNotNone(json.dumps(solution_dict))
+        self.assertIsNotNone(pickle.dumps(solution_dict))
         _, validator = get_schema_validator(schema_dir.joinpath("SolutionSchema.json"))
         # make sure is storable
         with tempfile.NamedTemporaryFile(mode="w+") as t:
@@ -374,21 +378,31 @@ class SerializationTests(unittest.TestCase):
         for obstacle in self.obstacles:
             as_json = json.dumps(obstacle.to_json_data())
             from_json = json.loads(as_json)
-            new = Obstacle.Obstacle.from_json_data(dict(
+            as_pickle = pickle.dumps(obstacle.to_json_data())
+            from_pickle = pickle.loads(as_pickle)
+            new_json = Obstacle.Obstacle.from_json_data(dict(
                 collision=from_json['collision'],
                 package_dir=package_dir,
                 ID=from_json['ID'],
                 name=from_json['name']))
+            new_pickle = Obstacle.Obstacle.from_json_data(dict(
+                collision=from_pickle['collision'],
+                package_dir=package_dir,
+                ID=from_pickle['ID'],
+                name=from_pickle['name']))
             new_from_string = Obstacle.Obstacle.from_json_string(obstacle.to_json_string())
-            self.assertIs(type(new), type(obstacle))
+            self.assertIs(type(new_json), type(obstacle))
+            self.assertIs(type(new_pickle), type(obstacle))
             self.assertIs(type(new_from_string), type(obstacle))
-            new_dict = new.__dict__.copy()
+            new_dict_json = new_json.__dict__.copy()
+            new_dict_pickle = new_pickle.__dict__.copy()
             newfs_dict = new_from_string.__dict__.copy()
             comp_dict = obstacle.__dict__.copy()
             for key in comp_dict:
                 if key in ('_children', 'collision'):
                     continue
-                np_test.assert_array_equal(new_dict[key], comp_dict[key])
+                np_test.assert_array_equal(new_dict_json[key], comp_dict[key])
+                np_test.assert_array_equal(new_dict_pickle[key], comp_dict[key])
                 np_test.assert_array_equal(newfs_dict[key], comp_dict[key])
 
     def test_serial_module_assembly(self):
@@ -405,15 +419,26 @@ class SerializationTests(unittest.TestCase):
         for tolerance in self.tolerances:
             as_json = json.dumps(tolerance.to_projection())
             from_json = json.loads(as_json)
-            new = Tolerance.AlwaysValidTolerance()
+            as_pickle = pickle.dumps(tolerance.to_projection())
+            from_pickle = pickle.loads(as_pickle)
+            new_json = Tolerance.AlwaysValidTolerance()
+            new_pickle = Tolerance.AlwaysValidTolerance()
             for proj, val in zip(from_json['toleranceProjection'], from_json['tolerance']):
-                new = new + Tolerance.ToleranceBase.from_projection(proj, val)
-            self.assertIs(type(new), type(tolerance))
+                new_json = new_json + Tolerance.ToleranceBase.from_projection(proj, val)
+            self.assertIs(type(new_json), type(tolerance))
             for key in tolerance.__dict__:
                 if isinstance(tolerance.__dict__[key], np.ndarray):
-                    np_test.assert_array_equal(new.__dict__[key], tolerance.__dict__[key])
+                    np_test.assert_array_equal(new_json.__dict__[key], tolerance.__dict__[key])
                 else:
-                    self.assertEqual(new.__dict__[key], tolerance.__dict__[key])
+                    self.assertEqual(new_json.__dict__[key], tolerance.__dict__[key])
+            for proj, val in zip(from_pickle['toleranceProjection'], from_pickle['tolerance']):
+                new_pickle = new_pickle + Tolerance.ToleranceBase.from_projection(proj, val)
+            self.assertIs(type(new_pickle), type(tolerance))
+            for key in tolerance.__dict__:
+                if isinstance(tolerance.__dict__[key], np.ndarray):
+                    np_test.assert_array_equal(new_pickle.__dict__[key], tolerance.__dict__[key])
+                else:
+                    self.assertEqual(new_pickle.__dict__[key], tolerance.__dict__[key])
 
     def test_wrap_monolithic_robot(self):
         panda = self.robot
@@ -450,6 +475,23 @@ class SerializationTests(unittest.TestCase):
         self.assertLess(pin_geometry_models_functionally_equal(modular_complex, fresh_robot, iterations=1000), 0.01)
         self.assertTrue(pin_geometry_models_structurally_equal(modular_complex.visual, fresh_robot.visual))
 
+    def test_urdf2json(self):
+        assembly = ModuleAssembly.from_monolithic_robot(self.robot, self.panda_urdf)
+        db_data = assembly.db.to_json_data()
+        reconstructed_db = ModulesDB.from_json_data(db_data, self.panda_assets, "panda")
+        assembly_from_json = ModuleAssembly.from_serial_modules(reconstructed_db, ("panda",))
+        with tempfile.NamedTemporaryFile() as f:
+            assembly.to_urdf(write_to=Path(f.name))
+            f.flush()
+            assembly_from_urdf = ModuleAssembly.from_monolithic_robot(
+                Robot.PinRobot.from_urdf(Path(f.name), self.panda_assets))
+
+        for recreated_assembly in (assembly_from_json, assembly_from_urdf):
+            self.assertTrue(pin_models_functionally_equal(self.robot.model, recreated_assembly.robot.model))
+            self.assertTrue(robot_io_equal(self.robot, recreated_assembly.robot))
+            self.assertTrue(pin_geometry_models_structurally_equal(self.robot.visual, recreated_assembly.robot.visual))
+            self.assertLess(pin_geometry_models_functionally_equal(self.robot, recreated_assembly.robot, 1000), 0.01)
+
 
 if __name__ == '__main__':
     unittest.main()
