@@ -14,12 +14,13 @@ from timor.task import Constraints, CostFunctions, Goals, Solution, Task, Tolera
 from timor.task.CostFunctions import QDist
 from timor.task.Solution import SolutionHeader, SolutionTrajectory
 from timor.task.Task import TaskHeader
-from timor.utilities import prebuilt_robots, spatial
+from timor.utilities import prebuilt_robots, spatial, visualization
 from timor.utilities.file_locations import get_test_tasks, robots
 from timor.utilities.prebuilt_robots import get_six_axis_assembly
 from timor.utilities.tolerated_pose import ToleratedPose
 from timor.utilities.trajectory import Trajectory
 from timor.utilities.transformation import Transformation
+from timor.utilities.visualization import clear_visualizer
 
 
 class TestSolution(unittest.TestCase):
@@ -253,7 +254,6 @@ class TestSolution(unittest.TestCase):
         pause_goal = Goals.Pause(ID='pause', duration=pause_duration,
                                  constraints=[Constraints.JointAngles(robot.joint_limits)])
 
-        Solution.SolutionBase.t_resolution = float('inf')  # Monkey patch to being able testing invalid solutions
         for _ in range(10):
             bad_trajectory = Trajectory(t=dt, q=np.random.random(q_array_shape))
             t_start = random.choice(tuple(bad_trajectory.t[bad_trajectory.t < max(bad_trajectory.t) - pause_duration]))
@@ -483,20 +483,60 @@ class TestSolution(unittest.TestCase):
         self.assertFalse(follow_goal_with_constraint._valid(
             invalid_sol, invalid_sol.t_goals["0"], len(invalid_sol.trajectory)))
 
+    def test_solution_derived_properties(self, num_test=10, epsilon=1e-12):
+        assembly = get_six_axis_assembly()  # Use IMPROV with significant joint inertia and friction
+        for _ in range(num_test):
+            qs = np.asarray((assembly.robot.random_configuration(), assembly.robot.random_configuration()))
+            sol = SolutionTrajectory(Trajectory.from_mstraj(qs, 0.01, 1., 1.), SolutionHeader("tmp"),
+                                     Task.Task(TaskHeader("tmp")), assembly, CostFunctions.RobotMass(),
+                                     assembly.robot.placement)
+
+            np_test.assert_array_less(np.abs(sol.link_side_torques), np.abs(sol.torques) + epsilon)
+            np_test.assert_array_less(sol._get_torques(False, True), sol.torques + epsilon)
+            np_test.assert_array_less(sol._get_torques(True, False), sol.torques + epsilon)
+            np_test.assert_array_less(sol.get_power(False, False), sol.get_power() + epsilon)
+            self.assertLess(sol.get_mechanical_energy(False, False), sol.get_mechanical_energy() + epsilon)
+
     def test_task_visuals(self):
         """Needs manual inspection for the plots"""
         robot = Robot.PinRobot.from_urdf(self.panda_urdf, robots['panda'].parent)
+        robot.move(Transformation.from_translation((-2., -2., .5)))
         q0 = pin.neutral(robot.model)
         q1 = pin.randomConfiguration(robot.model)
         q2 = pin.randomConfiguration(robot.model)
+        viz = None
+        color_viz = None
+
+        empty_task = Task.Task.empty()
+        empty_solution = Solution.SolutionTrajectory.empty()
+        empty_viz = empty_task.visualize()
+        empty_viz = empty_task.visualize(empty_viz, robots=robot)
+        empty_solution.visualize(empty_viz)
+
         for description in self.task_files:
             robot.update_configuration(q0)
             task = Task.Task.from_json_file(description, self.asset_dir)
             robot.update_configuration(q1)
-            viz = task.visualize(robots=robot)  # Machine should be red
+            viz = task.visualize(viz, robots=robot, center_view=False)  # Should have view centered on origin
+            clear_visualizer(viz)
+            viz = task.visualize(viz, robots=robot, center_view=True)  # Should have view including more of the robot
+            clear_visualizer(viz)
+            viz = task.visualize(viz, center_view=True)  # Should have view including base constraint, e.g. PTP_1
+            clear_visualizer(viz)
             robot.update_configuration(q2)
             self.assertIs(robot.data, viz.data)
 
+            assembly = get_six_axis_assembly()
+            assembly.robot.update_configuration(assembly.robot.random_configuration())
+            color_viz = task.visualize(color_viz, robots=assembly.robot)
+            visualization.color_visualization(color_viz, assembly)
+
+            id_cmap = {mid: np.random.rand(4) for mid in assembly.original_module_ids}
+            visualization.color_visualization(color_viz, assembly, color_map=id_cmap)
+
+            custom_id_cmap = {m.id: np.random.rand(4) for m in assembly.module_instances}
+            visualization.color_visualization(color_viz, assembly, color_map=custom_id_cmap)
+
 
 if __name__ == '__main__':
     unittest.main()
